#include "dialog.h"
#include "backgnd.h"
#include <dcopclient.h>
#include <klocale.h>
#include <kbuttonbox.h>
#include <kseparator.h>
#include <kapplication.h>
#include <kconfig.h>
#include <qlayout.h>
#include <qpushbutton.h>

#include <X11/X.h>
#include <X11/Xlib.h>

MyBackgroundDialog::MyBackgroundDialog(const QString &fileStr, QWidget *parent,
                                       const char *name)
    : QDialog(parent, name, true)
{
    int screen_number = 0;
    if (qt_xdisplay())
        screen_number = DefaultScreen(qt_xdisplay());
    QCString configname;
    if(screen_number == 0)
        configname = "kdesktoprc";
    else
        configname.sprintf("kdesktop-screen-%drc", screen_number);
    config = new KConfig(configname, false, false);


    QVBoxLayout *layout = new QVBoxLayout(this, 5, 5);
    page = new Backgnd(this, config);
    page->load();
    layout->addWidget(page);
    layout->addWidget(new KSeparator(Qt::Horizontal, this));

    KButtonBox *bbox = new KButtonBox(this);

    connect(bbox->addButton(i18n("Defaults")), SIGNAL(clicked()), this,
            SLOT(slotDefaults()));
    bbox->addStretch(1);
    connect(bbox->addButton(i18n("OK")), SIGNAL(clicked()), this,
            SLOT(slotApply()));
    connect(bbox->addButton(i18n("Cancel")), SIGNAL(clicked()), this,
            SLOT(reject()));

    layout->addWidget(bbox);

    page->setFile(fileStr);
    setCaption(i18n("Set Background Wallpaper"));
}

MyBackgroundDialog::~MyBackgroundDialog()
{
    delete config;
}

void MyBackgroundDialog::slotDefaults()
{
    page->defaults();
}

void MyBackgroundDialog::slotApply()
{
    page->save();
    // reconfigure kdesktop. kdesktop will notify all clients
    DCOPClient *client = kapp->dcopClient();
    if (!client->isAttached())
	client->attach();

    int screen_number = 0;
    if (qt_xdisplay())
	screen_number = DefaultScreen(qt_xdisplay());
    QCString appname;
    if (screen_number == 0)
	appname = "kdesktop";
    else
	appname.sprintf("kdesktop-screen-%d", screen_number);

    client->send(appname, "KBackgroundIface", "configure()", "");
    accept();
}

void MyBackgroundDialog::slotChanged()
{
}

