#ifndef __IMAGELOADER_H
#define __IMAGELOADER_H

#include <qimage.h>
#include <qobject.h>
#include "imageviewer.h"

class ImageLoader : public QObject
{
public:
    ImageLoader(QImage &, ImageView *viewer){view=viewer;}
    virtual ~ImageLoader(){;}
    virtual bool readData(char *buffer, int size) = 0;
    virtual bool error() = 0;
    void imageUpdated(int start_row, int end_row){view->updateScanlines(start_row, end_row);}
protected:
    ImageView *view;
};

class PNGImageLoader : public ImageLoader
{
public:
    PNGImageLoader(QImage &image, ImageView *viewer);
    ~PNGImageLoader();
    bool readData(char *buffer, int size);
    bool error();
};

class JPEGPrivate;

class JPEGImageLoader : public ImageLoader
{
public:
    enum{Init, readHeader, startDecompress, decompressStarted, consumeInput,
        prepareOutputScan, doOutputScan, readDone, invalid} state;

    JPEGImageLoader(QImage &image, ImageView *viewer);
    ~JPEGImageLoader();
    bool readData(char *buffer, int size);
    int processData(char *buffer, int size);
    bool error();
protected:
    JPEGPrivate *d;
};

class GIFImageLoader : public ImageLoader
{
public:
    enum State{Header, LogicalScreenDescriptor, GlobalColorMap, LocalColorMap,
	Introducer, ImageDescriptor, TableImageLZWSize, ImageDataBlockSize,
	ImageDataBlock, ExtensionLabel, GraphicControlExtension,
        ApplicationExtension, NetscapeExtensionBlockSize,
        NetscapeExtensionBlock, SkipBlockSize, SkipBlock, Done, Error};
    enum Disposal{NoDisposal, DoNotChange, RestoreBackground, RestoreImage};
    enum{ max_lzw_bits=12 };

    GIFImageLoader(QImage &image, ImageView *viewer);
    ~GIFImageLoader();
    bool readData(char *buffer, int size);
    bool error();
private:
    void fillRect(int x, int y, int w, int h, QRgb col);
    void disposePrevious();
    void nextY();
    QRgb color(unsigned char index) const;

    QRgb *globalcmap, *localcmap;
    QImage backingstore;
    unsigned char hold[16];
    bool gif89;
    int count, ccount, expectcount, gncols, lncols, ncols, lzwsize;
    bool lcmap;
    int swidth, sheight, left, top, right, bottom;
    Disposal disposal;
    bool disposed;
    int trans_index, bgcol, interlace, accum, bitcount;
    bool gcmap;

    int code_size, clear_code, end_code, max_code_size, max_code;
    int firstcode, oldcode, incode;
    short table[2][1<< max_lzw_bits];
    short stack[(1<<(max_lzw_bits))*2];
    short *sp;
    bool needfirst;
    int x, y;
    int frame;
    bool out_of_bounds;
    bool digress;
    QImage *img;
    QRect updateR;
    State state;
};

class BMP_FILEHDR;
class BMP_INFOHDR;

class BMPImageLoader : public ImageLoader
{
public:
    enum State{FileHeader, InfoHeader, InfoHeaderOld, ColorMap, BitFields,
        SkipToColorMap, SkipToImage, ImageBitmap, Image4Bit, Image4BitRLE,
        Image8Bit, Image8BitRLE, Image16Bit, Image24Bit, Image32Bit, Error,
        End};

    BMPImageLoader(QImage &image, ImageView *viewer);
    ~BMPImageLoader();
    bool readData(char *buffer, int size);
    int processData(char *buffer, int size);
    bool error();
protected:
    int calc_shift(int mask);
    void swapColormap();
    void createImage();
    void calcImageType();

    State state;
    bool hasError;
    unsigned char *hold;
    int headerSize, count, position;
    BMP_FILEHDR *bf;
    BMP_INFOHDR *bi;

    int ncols, depth, currentX, currentY, bpl;
    unsigned int *data;
    unsigned char *byteData;
    int w, h, nbits, t, comp, temp;
    int red_mask, green_mask, blue_mask;
    int red_shift, green_shift, blue_shift;
    int red_scale, green_scale, blue_scale;
    int colormapSize;
    bool inEsc;
    QImage *img;

};


#endif

