#ifndef __IMAGEVIEWER_H
#define __IMAGEVIEWER_H
#include <qscrollview.h>
#include <qfile.h>
#include <kio/job.h>

class ImageLoader;

class LocalImagePreview : public QScrollView
{
public:
    LocalImagePreview();
    ~LocalImagePreview();
    void setFile(const QString &fn);
protected:
    void drawContents(QPainter *p, int x, int y, int w, int h);

    QPixmap *pix;
    QString filename;
};

class ImageView : public QScrollView
{
    Q_OBJECT
public:
    ImageView(QWidget *parent=0, const char *name=0);
    ~ImageView();
    bool openURL(const KURL &url, const QString &mimetype, bool reload);
    bool openFile(const QString &file);
    bool closeURL();
    void updateScanlines(int start_row, int end_row);
    KURL& currentURL(){return(curURL);}
    QString& currentMimetype(){return(mimeStr);}
    QString& currentFilename(){return(fileStr);}
    QStringList* hotListPaths(){return(&hotPathList);}
    QStringList* hotListNames(){return(&hotNameList);}
    QPixmap* pixmap(){return(pix);}
    bool isFinished(){return(finished);}
    bool isLocal(){return(isLocalFile);}
    void save(const QString &fn);
    void reloadConfig();
signals:
    void started(KIO::Job *);
    void completed();
    void canceled(const QString &);
    void rightClick(const QPoint &globalPos);
    void title(const QString &);
    void status(const QString &);
    void copyFinished();
protected slots:
    void slotReadData(KIO::Job *, const QByteArray &input);
    void slotTotalSize(KIO::Job *, KIO::filesize_t data_size);
    void slotMimetype(KIO::Job *, const QString &type);
    void slotResult(KIO::Job *);

    void slotCopyJobResult(KIO::Job *);
protected:
    void drawContents(QPainter *p, int x, int y, int w, int h);
    void loadConfig();
    void reset();
    void checkForDupes(const QString &filename);
    void contentsMousePressEvent(QMouseEvent *ev);
    void contentsMouseMoveEvent(QMouseEvent *ev);

    bool isLocalFile;
    bool finished;
    bool hasMimetype;
    bool hasSizeCaption;
    bool initialized;
    QString mimeStr;
    QString fileStr;
    KURL curURL;
    QImage *image;
    QPixmap *pix;
    KIO::TransferJob *job;
    ImageLoader *loader;
    QFile *buffer;
    bool hasError;

    // This is for the PixiePlus Hotlist duplicates
    int start_y, textWidth, currentHighlight;
    QStringList hotPathList, hotNameList;
    QStringList dupePathList, dupeNameList;
    QValueList<QRect> dupeRectList;
    QRect totalLinkRect;
    bool hasHandCursor;
    QColor linkColor;
    QFont linkFont;
    QFontMetrics *fm;
    LocalImagePreview *preview;
};


#endif

