#include "mosfetimage_part.h"
#include "hotlistdialog.h"

#include <klocale.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kimageio.h>
#include <kinstance.h>
#include <kaction.h>
#include <kfiledialog.h>
#include <kprinter.h>
#include <kparts/genericfactory.h>
#include <kxmlguifactory.h>
#include <kaction.h>
#include <kaboutdata.h>
#include <kstandarddirs.h>
#include <qtimer.h>
#include <qiconset.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qpainter.h>
#include <qprocess.h>

static const char *description = I18N_NOOP("Mosfet Image Loader For Konqueror");
static const char *version     = "0.2";

typedef KParts::GenericFactory<MosfetImagePart> MosfetImageFactory;
K_EXPORT_COMPONENT_FACTORY(libmosfetimage, MosfetImageFactory);

MosfetImagePart::MosfetImagePart(QWidget *parentWidget,
                                 const char *widgetName,
                                 QObject *parent, const char *name,
                                 const QStringList &)
    : KParts::ReadOnlyPart(parent, name)
{
    qWarning("In part constructor");
    setInstance(MosfetImageFactory::instance());
    inBackground = false;
    wallpaperProc = NULL;

    KImageIO::registerFormats();
    widget = new ImageView(parentWidget, widgetName);
    setWidget(widget);
    // since the widget handles KIO we need to resend all it's KIO signal
    // stuff :P Probably should do KIO in this class...
    connect(widget, SIGNAL(started(KIO::Job *)), this,
            SLOT(slotViewerStarted(KIO::Job *)));
    connect(widget, SIGNAL(canceled(const QString &)), this,
            SLOT(slotViewerCanceled(const QString &)));
    connect(widget, SIGNAL(completed()), this,
            SLOT(slotViewerCompleted()));
    connect(widget, SIGNAL(rightClick(const QPoint &)), this,
            SLOT(slotViewerPopup(const QPoint &)));
    connect(widget, SIGNAL(title(const QString &)), this,
            SLOT(slotTitle(const QString &)));
    connect(widget, SIGNAL(status(const QString &)), this,
            SLOT(slotStatus(const QString &)));
    connect(widget, SIGNAL(copyFinished()), this,
            SLOT(slotFinishedCopy()));

    extension = new MosfetImageBrowserExtension(this);
}

MosfetImagePart::~MosfetImagePart()
{
    qWarning("In part destructor");
}

KAboutData* MosfetImagePart::createAboutData()
{

    KAboutData *about = new KAboutData("mosfetimage", I18N_NOOP("mosfetimage"),
                                       version, description,
                                       KAboutData::License_BSD,
                                       "(C) 2003 Daniel M. Duley");
    return about;
}

void MosfetImagePart::guiActivateEvent(KParts::GUIActivateEvent *ev)
{
    emit extension->enableAction("print", widget->isFinished() &&
                                 widget->pixmap());
    ReadOnlyPart::guiActivateEvent(ev);
}

bool MosfetImagePart::openURL(const KURL &url)
{
    if(url.isMalformed()){
        qWarning("Malformed url!");
        return(false);
    }
    m_url = url;

    emit extension->enableAction("print", false);
    qWarning("Got url %s, Got mimetype %s",
             url.url().latin1(), extension->urlArgs().serviceType.latin1());
    return(widget->openURL(url, extension->urlArgs().serviceType,
                           extension->urlArgs().reload));
}

bool MosfetImagePart::closeURL()
{
    widget->closeURL();
    emit extension->enableAction("print", widget->isFinished() &&
                                 widget->pixmap());
    return(ReadOnlyPart::closeURL());
}

void MosfetImagePart::slotViewerStarted(KIO::Job *job)
{
    emit started(job);
}

void MosfetImagePart::slotViewerCanceled(const QString &str)
{
    emit canceled(str);
    emit extension->enableAction("print", widget->isFinished() &&
                                 widget->pixmap());
}

void MosfetImagePart::slotViewerCompleted()
{
    emit completed();
    emit extension->enableAction("print", widget->isFinished() &&
                                 widget->pixmap());
}

class PopupGUIClient : public KXMLGUIClient
{
public:
    PopupGUIClient(KInstance *inst, const QString &doc)
    {
        setInstance(inst);
        setXML(doc);
    }
};

void MosfetImagePart::slotViewerPopup(const QPoint &pos)
{
    // Dynamically create the XML document string because emit popupMenu()
    // cannot have ActionLists in the client. It builds the GUI after being
    // emitted and we need the hotlist to be dynamic.
    QString docStr;
    docStr += "<!DOCTYPE kpartgui SYSTEM \"kpartgui.dtd\">\n";
    docStr += "<kpartgui name=\"mosfetimage_popup\" version=\"1\">\n";
    docStr += "    <Menu name=\"popupmenu\">\n";
    docStr += "        <Separator />\n";
    docStr += "        <Action name=\"copylinklocation\"/>\n";
    docStr += "        <Action name=\"copyfilename\"/>\n";
    docStr += "        <Separator />\n";
    docStr += "        <Action name=\"setbackground\"/>\n";
    docStr += "        <Separator />\n";
    docStr += "        <Action name=\"configurehotlist\"/>\n";
    docStr += "        <Separator />\n";
    docStr += "        <Action name=\"saveimageas\"/>\n";
    docStr += "        <Separator />\n";

    // Do the hotlist
    QStringList *nameList = widget->hotListNames();
    QStringList::Iterator it;
    int i;
    for(it = nameList->begin(), i=0; it != nameList->end(); ++it, ++i){
        docStr += QString("        <Action name =\"hotlist")
            += QString::number(i) += "\"/>\n";
    }
    // finish up the document string
    docStr += "    </Menu>\n";
    docStr += "</kpartgui>";

    KXMLGUIClient *popupGUIClient = new PopupGUIClient(instance(), docStr);

    (void)new KAction(i18n("Copy Image Location" ),
                      BarIcon("editcopy", 16), 0, this,
                      SLOT(slotCopyLocation()),
                      popupGUIClient->actionCollection(),
                      "copylinklocation");
    (void)new KAction(i18n("Copy Image Filename" ),
                      BarIcon("editcopy", 16), 0, this,
                      SLOT(slotCopyFilename()),
                      popupGUIClient->actionCollection(),
                      "copyfilename");

    KAction *action = new KAction(i18n("Set As Wallpaper Background..."),
                                  BarIcon("configure", 16), 0, this,
                                  SLOT(slotBackground()),
                                  popupGUIClient->actionCollection(),
                                  "setbackground");
    action->setEnabled(widget->isFinished());
    action = new KAction(i18n("Configure Hotlist Groups..."),
                         BarIcon("configure", 16), 0, this,
                         SLOT(slotConfigureHotlist()),
                         popupGUIClient->actionCollection(),
                         "configurehotlist");
    action->setEnabled(widget->isFinished());
    action = new KAction(i18n("Save Image As..."),
                         BarIcon("filesave", 16), 0, this,
                         SLOT(slotSaveAs()),
                         popupGUIClient->actionCollection(),
                         "saveimageas");
    action->setEnabled(widget->isFinished());
    // hotlist groups
    QString idStr;
    QSignalMapper *hotlistMap = new QSignalMapper(this);
    connect(hotlistMap, SIGNAL(mapped(int)), this, SLOT(slotHotList(int)));
    for(it = nameList->begin(), i=0; it != nameList->end(); ++it, ++i){
        idStr = "hotlist";
        idStr += QString::number(i);
        action = new KAction(i18n("Copy to ") + (*it),
                             BarIcon("editcopy", 16), 0, hotlistMap,
                             SLOT(map()), popupGUIClient->actionCollection(),
                             idStr.latin1());
        hotlistMap->setMapping(action, i);
        action->setEnabled(widget->isFinished());
    }
    extension->popupMenu(popupGUIClient, pos, widget->currentURL(),
                         widget->currentMimetype(), S_IFDIR);
    delete popupGUIClient;
    delete hotlistMap;
}

void MosfetImagePart::slotTitle(const QString &str)
{
    emit setWindowCaption(str);
}

void MosfetImagePart::slotStatus(const QString &str)
{
    emit setStatusBarText(str);
}

void MosfetImagePart::slotCopyLocation()
{
    QApplication::clipboard()->setText(widget->currentURL().url(),
                                       QClipboard::Selection);
}

void MosfetImagePart::slotCopyFilename()
{
    QApplication::clipboard()->setText(widget->currentFilename(),
                                       QClipboard::Selection);
}

void MosfetImagePart::slotSaveAs()
{
    QString fn = KFileDialog::getSaveFileName(widget->currentFilename(),
                                              QString::null, widget);
    if(!fn.isEmpty())
        widget->save(fn);
}

void MosfetImagePart::slotHotList(int id)
{
    QString fn = *(widget->hotListPaths()->at(id));
    if(!fn.isEmpty()){
        fn += "/" + widget->currentFilename();
        widget->save(fn);
    }
    else
        qWarning("Invalid hotlist id!");
}

void MosfetImagePart::slotConfigureHotlist()
{
    HotListDialog dlg(widget);
    if(dlg.exec() == QDialog::Accepted)
        widget->reloadConfig();
}

void MosfetImagePart::slotBackground()
{
    if(!wallpaperProc){
        inBackground = true;
        widget->save(locateLocal("wallpaper", widget->currentFilename()));
    }
    else{
        KMessageBox::sorry(widget, i18n("Please close current wallpaper window first!"));
    }
}

void MosfetImagePart::slotFinishedCopy()
{
    if(inBackground){
        qWarning("Finished wallpaper copy");
        inBackground = false;
        wallpaperProc = new QProcess(this);
        wallpaperProc->addArgument("mosfetwallpaper");
        wallpaperProc->addArgument("--wallpaper");
        wallpaperProc->addArgument(widget->currentFilename());
        connect(wallpaperProc, SIGNAL(processExited()), this,
                SLOT(slotWallpaperProcExited()));
        if(!wallpaperProc->start()){
            delete wallpaperProc;
            wallpaperProc = 0;
            KMessageBox::sorry(widget, i18n("Could not run wallpaper configuration utility!"));
        }
    }
}

void MosfetImagePart::slotWallpaperProcExited()
{
    if(wallpaperProc){
        qWarning("Wallpaper configure finished.");
        delete wallpaperProc;
        wallpaperProc = 0;
    }
}

MosfetImageBrowserExtension::MosfetImageBrowserExtension(MosfetImagePart *parent)
    : KParts::BrowserExtension(parent, "MosfetImageBrowserExtension")
{
    imagePart = parent;
    setURLDropHandlingEnabled(false);
}

MosfetImageBrowserExtension::~MosfetImageBrowserExtension()
{
}

int MosfetImageBrowserExtension::xOffset()
{
    return(imagePart->widget->contentsX());
}

int MosfetImageBrowserExtension::yOffset()
{
    return(imagePart->widget->contentsY());
}

void MosfetImageBrowserExtension::print()
{
    if(!imagePart->widget->isFinished()){
        qWarning("Print called before image loaded!");
        return;
    }
    if(!imagePart->widget->pixmap()){
        qWarning("Print called w/ no pixmap!");
        return;
    }
    KPrinter printer;
    printer.setFullPage(true);
    if(printer.setup(imagePart->widget)){
        QPainter p;
        p.begin(&printer);
        p.drawPixmap(0, 0, *imagePart->widget->pixmap());
        p.end();
    }
}

void MosfetImageBrowserExtension::reparseConfiguration()
{
}

